# Generated by Django 3.2.8 on 2021-12-27 14:53

import django.contrib.postgres.fields
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(max_length=1024, verbose_name='Domain')),
                ('tracking', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=128), blank=True, default=list, size=None, verbose_name='Tracking codes stack')),
                ('message', models.TextField(blank=True, default='', verbose_name='Message')),
                ('detail', models.JSONField(blank=True, default=dict, verbose_name='Detail')),
                ('created_at', models.DateTimeField(verbose_name='Created at')),
            ],
            options={
                'verbose_name': 'Log entry',
                'verbose_name_plural': 'Log entries',
                'ordering': ('-created_at', 'domain'),
            },
        ),
    ]
